/*
 *      INSTALL.CMD Generic MAC Wrapper Driver - Copyright Willibald Meyer 2005
 *      Installation for Generic MAC Wrapper Driver - Copyright C.Langanke 2005
 *
 *      Syntax: INSTALL
 *
 *      This program installs the Generic MAC Wrapper Driver for usage
 *      with the MPTS.
 *
 */
/* The first comment is used as online help text */

 SIGNAL ON HALT

 TitleLine = STRIP(SUBSTR(SourceLine( 2), 3));
 PARSE VAR TitleLine CmdName'.CMD 'Info
 Title     = Info
 Subtitle  = STRIP(SUBSTR(SourceLine( 3), 3)); 

 env          = 'OS2ENVIRONMENT';
 TRUE         = (1 = 1);
 FALSE        = (0 = 1);
 CrLf         = '0d0a'x;
 Redirection  = '> NUL 2>&1';
 '@ECHO OFF'

 /* OS/2 Error codes */
 ERROR.NO_ERROR           =   0;
 ERROR.INVALID_FUNCTION   =   1;
 ERROR.FILE_NOT_FOUND     =   2;
 ERROR.PATH_NOT_FOUND     =   3;
 ERROR.ACCESS_DENIED      =   5;
 ERROR.NOT_ENOUGH_MEMORY  =   8;
 ERROR.INVALID_FORMAT     =  11;
 ERROR.INVALID_DATA       =  13;
 ERROR.NO_MORE_FILES      =  18;
 ERROR.WRITE_FAULT        =  29;
 ERROR.READ_FAULT         =  30;
 ERROR.GEN_FAILURE        =  31;
 ERROR.SHARING_VIOLATION  =  32;
 ERROR.INVALID_PARAMETER  =  87;
 ERROR.ENVVAR_NOT_FOUND   = 203;

 GlobalVars = 'Title CmdName CrLf env TRUE FALSE Redirection ERROR.';
 SAY;

 /* eventually show help */
 ARG Parm .
 IF (POS('?', Parm) > 0) THEN
 DO
    rc = ShowHelp();
    EXIT(ERROR.INVALID_PARAMETER);
 END;

 call RxFuncAdd    'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
 call SysLoadFuncs

 /* dafault values */
 GlobalVars = GlobalVars 'CallDir AppName';
 rc = ERROR.NO_ERROR;

 CallDir  = GetCallDir();
 PARSE VALUE TRANSLATE( VALUE( 'PATH',,env)) WITH '\OS2;' -2 BootDrive +2;

 DO 1

    SAY Title;
    SAY SubTitle;
    SAY;

    /* determine some enrironment parameters */
    ConfigFile = BootDrive'\config.sys';
    MPTSDir = GetProgramPath( 'MPTS.EXE');
    MacsDir = MPTSDir'\macs';
    NetBindExec = MPTSDir'\protocol\netbind.exe';

    InstallDir = BootDrive'\genmac';
    HelperExecutable = InstallDir'\driver\helperw.exe';
    UnlockExec = CallDir'\bin\unlock.exe';

    SAY 'GenMac installation directory:' InstallDir;
    SAY 'MPTS MAC driver directory    :' MacsDir;
    SAY

    /* copy files to default program directory */
    IF (TRANSLATE( InstallDir) \= TRANSLATE( CallDir)) THEN
    DO
       /* unlock helper executable, if exists */
       IF (FileExist( HelperExecutable)) THEN
       DO
          SAY '- unlock helper executable';
          '' UnlockExec HelperExecutable Redirection;
       END;

       SAY '- copying files to install dir'
       rc = CopyFiles( CallDir'\*', InstallDir, TRUE);
    END;

    /* copy NIF files */
    SAY '- copying driver files to MPTS'
    rc = CopyFiles( CallDir'\driver\genm32*.os2', MacsDir, FALSE);
    rc = CopyFiles( CallDir'\driver\genm32*.msg', MPTSDir, FALSE);

    /* copy NIF files */
    SAY '- copying description files to MPTS'
    rc = CopyFiles( CallDir'\nif\*', MacsDir, TRUE);

    /* install helper executable file */
    SAY '- updating system configuration'
    rc = InstallHelper( ConfigFile, HelperExecutable, NetBindExec);

    SAY;
    SAY '  Please configure the driver corresponding to your hardware within MPTS.';
    SAY '  After that, in order to load the new or updated driver(s),';
    SAY '  REBOOT YOUR SYSTEM.';
 END;

 EXIT( rc);

/* ------------------------------------------------------------------------- */
HALT:
 SAY 'Abbruch durch Benutzer.';
 EXIT(ERROR.GEN_FAILURE);

/* ------------------------------------------------------------------------- */
ShowHelp: PROCEDURE EXPOSE (GlobalVars)

 /* show title */
 SAY Title;
 SAY;

 PARSE SOURCE . . ThisFile

 /* skip header */
 DO i = 1 TO 3
    rc = LINEIN(ThisFile);
 END;

 /* show help text */
 ThisLine = LINEIN(Thisfile);
 DO WHILE (ThisLine \= ' */')
    SAY SUBSTR(ThisLine, 7);
    ThisLine = LINEIN(Thisfile);
 END;

 /* close file */
 rc = LINEOUT(Thisfile);

 RETURN('');

/* ------------------------------------------------------------------------- */
GetCalldir: PROCEDURE
PARSE SOURCE . . CallName
 CallDir = FILESPEC('Drive', CallName)||FILESPEC('Path', CallName);
 RETURN(LEFT(CallDir, LENGTH(CallDir) - 1));

/* ------------------------------------------------------------------------- */
GetDrivePath: PROCEDURE
 PARSE ARG FileName

 FullPath = FILESPEC('D', FileName)||FILESPEC('P', FileName);
 IF (FullPath \= '') THEN
    RETURN(LEFT(FullPath, LENGTH(FullPath) - 1));
 ELSE
    RETURN('');

/* ------------------------------------------------------------------------- */
FileExist: PROCEDURE
 PARSE ARG FileName

 RETURN(STREAM(Filename, 'C', 'QUERY EXISTS') > '');

/* ========================================================================= */
GetProgramPath: PROCEDURE EXPOSE (GlobalVars);
 PARSE ARG ExeName

 ProgramPath = '';

 DO 1
   /* determine program fullname */
   ProgramPath = SysSearchPath( 'PATH', ExeName);
   IF (ProgramPath = '') THEN
      LEAVE;

   /* isolate path if desired */
      ProgramPath = LEFT( ProgramPath, LASTPOS( '\', ProgramPath) - 1);
 END;

 RETURN( ProgramPath);

/* ========================================================================= */
CopyFiles: PROCEDURE EXPOSE (GlobalVars);
 PARSE ARG SourceFiles, TargetDir, fIncludeSubDirs

 rc = ERROR.NO_ERROR;

 DO 1
    /* determine static part of source file path */
    SourceFiles = STREAM( SourceFiles, 'C', 'QUERY EXISTS');
    IF (SourceFiles = '') THEN
    DO
       SAY 'error: package files not found.';
       EXIT( ERROR.PATH_NOT_FOUND);
    END;
    BasePathLen = LASTPOS( '\', SourceFiles) + 1;

    /* search files */
    SearchOpts = 'FO';
    IF (fIncludeSubDirs) THEN
       SearchOpts = SearchOpts'S';
    rc = SysFileTree( SourceFiles, 'File.', SearchOpts);
    IF (rc \= 0) THEN
    DO
       SAY 'error in SysFileTree, reboot required!';
       EXIT( 99);
    END;

    /* copy file by file */
    DO i = 1 TO File.0
       SourceFile = File.i;
       TargetFile = TargetDir'\'SUBSTR( File.i, BasePathLen);
       TargetPath = LEFT( TargetFile, LASTPOS( '\', TargetFile) - 1);
       rc = MakePath( TargetPath);
       'COPY' SourceFile  TargetPath'\' ReDirection;
    END;

 END;

 RETURN( rc);

/* ========================================================================= */
MakePath: PROCEDURE
 PARSE ARG Path;

 DO 1
    /* tokenize path parts */
    PathNames = TRANSLATE( Path, ' ', '\');
    DO i = 2 TO WORDS( PathNames)
       ThisPath = LEFT( Path, WORDINDEX( PathNames, i) - 1 + LENGTH( WORD( PathNames, i)));
       rc = SysMkDir( ThisPath);
    END;

 END;

 RETURN( 0);

/* ========================================================================= */
ReadFile: PROCEDURE EXPOSE (GlobalVars);
 PARSE ARG File;

 Contents = '';

 DO 1
    FileSize = STREAM( File, 'C', 'QUERY SIZE');
    IF (FileSize = 0) THEN
       LEAVE;

    IF (STREAM( File, 'C', 'OPEN READ') \= 'READY:') THEN
       LEAVE;

    Contents = CHARIN( File, 1, FileSize);

    rcx = STREAM( File, 'C', 'CLOSE');
 END;

 RETURN( Contents);

/* ========================================================================= */
InstallHelper: PROCEDURE EXPOSE (GlobalVars);
 PARSE ARG ConfigFile, HelperExec, NetBindExec;

 DO 1

    /* create tmp file */
    TmpFile = SysTempFilename( VALUE('TMP',,env)'\gnminst.???');
    IF (STREAM( TmpFile, 'C', 'OPEN WRITE') \= 'READY:') THEN
    DO
       SAY 'error: cannot write' TmpFile;
       LEAVE;
    END;

    /* open config file */
    IF (STREAM( ConfigFile, 'C', 'OPEN READ') \= 'READY:') THEN
    DO
       SAY 'error: cannot read' ConfigFile;
       LEAVE;
    END;

    /* process file */
    NetBindExecUc = TRANSLATE( NetBindExec);
    HelperExecUc  = TRANSLATE( HelperExec);
    fNetBindFound = FALSE;
    fHelperAdded  = FALSE;


    /* always put helper app at first line of config.sys */
    rcx = LINEOUT( TmpFile, 'RUN='HelperExecUc);

    DO WHILE (LINES( ConfigFile))

       ThisLine = LINEIN( ConfigFile);
       fSkipLine = FALSE;

       /* tokenize line */
       PARSE VALUE TRANSLATE( ThisLine) WITH ThisStatement'='ThisValue
       ThisStatement = STRIP( ThisStatement);
       ThisValue     = STRIP( ThisValue);

       SELECT
          WHEN (ThisStatement = 'CALL') THEN
          DO
             /* if this is the netbind call, prepend run of helper */
             IF (ThisValue = NetBindExecUc) THEN
                fNetBindFound = TRUE;
          END;

          WHEN (ThisStatement = 'RUN') THEN
          DO
             ExecName = FILESPEC( 'N', WORD( ThisValue, 1));
             PARSE VAR ExecName ExeBaseName'.'ExeExtension;

             /* if this is a run for the helper, check to eliminate it */
             /* also eliminate Willibalds special versions */
             IF ((WORDPOS( ExeBaseName, 'HELPERW HELPERW1 HELPER1') > 0) &,
                 (ExeExtension = 'EXE')) THEN
                fSkipLine = TRUE;
          END;

          OTHERWISE NOP;

       END;

       IF (\fSkipLine) THEN
          rcx = LINEOUT( TmpFile, ThisLine);
    END;
    rcx = STREAM( ConfigFile, 'C', 'CLOSE');
    rcx = STREAM( TmpFile,    'C', 'CLOSE');

    /* ------------------------------------------ */

    /* if nothing changed, remove tmp file and exit */
    OldConfig = ReadFile( ConfigFile);
    NewConfig = ReadFile( TmpFile);
    IF (OldConfig = NewConfig) THEN
    DO
       rcx = SysFileDelete( TmpFile);
       SAY '  no configuration change necessary.';
       LEAVE;
    END;

    /* get original name of config file */
    rc = SysFileTree( ConfigFile, 'File.', 'FO');
    ConfigFile = File.1;

    /* create backup copy */
    DO i = 1 TO 999
       BackupFile = OVERLAY( RIGHT( i, 3, '0'), ConfigFile, LASTPOS( '.', ConfigFile) + 1);
       IF (\FileExist( BackupFile)) THEN
          LEAVE;
    END;
    'COPY' ConfigFile BackupFile Redirection;
    SAY '  modified' ConfigFile', created backup' BackupFile

    /* copy over updated version */
    'COPY' TmpFile ConfigFile Redirection;
    rcx = SysFileDelete( TmpFile);
 END;

 RETURN( 0);

